#
# composer.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@loveruby.net>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU General Public License version 2 or later.
#


module Delta

  class NewMailComposer_

    def initialize( mbox, from, body )
      @mbox = mbox
      @from = from
      @body = body
    end

    def exec( *to )
      m = create()
      m.to = to
      m.subject = ''
      m.body = @body
      m.write_back
      m.port
    end

    def create
      port = @mbox.new_port {|port|
        port.wopen { ; }
      }
      m = TMail::Mail.new( port )
      m.from = @from
      m['X-Mailer'] = "Delta #{Version}"
      m
    end
  
  end

  class NewMailComposer < NewMailComposer_
  end


  class ReplyMailComposer_

    def initialize( nm, &block )
      @newmail = nm
      @proc = block
      @re = /\A(\s*\[[^]]+\])?(?:\s*Re:)+\s*/i
    end

    def exec( port )
      orig = TMail::Mail.new(port)
      m = @newmail.create
      m.to_addrs = orig.reply_addresses
      m.subject = 'Re: ' + orig.subject('').sub(re, '')
      begin
        m.body = @proc.call(m)
      rescue StandardError, ScriptError => err
        m.body = ''
      end
      m.write_back
      m.port
    end
  
  end

  class ReplyMailComposer < ReplyMailComposer_
  end

end
