#
# menumaker.rb
#
#   Copyright (c) 2001,2002 Minero Aoki <aamine@loveruby.net>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU General Public License version 2 or later.
#

module Delta

  module MenuActions

    module_function

    def action?( name )
      respond_to? name
    end

    def write( msg )
      $stderr.puts msg
    end

    def trash
      count = move_selection_to( Delta.mailer.current_profile.trash )
      case count
      when 0 then write 'no mail selected'
      when 1 then write 'trashed'
      else        write "trashed #{count} mails"
      end
    end

    def move_selection_to( dest )
      Delta.mailer.views.current.move_selection_to dest
    end

    def update_mbox
      Delta.mailer.views.current.update
    end

    def reload_mbox
      Delta.mailer.views.current.reload
    end

    def clear_trash
      n = Delta.mailer.current_profile.trash.clear
      write "cleared #{n} mails from trash"
    end

    def refile
      Delta.mailer.refile
    end

    def read_ahead
      Delta.mailer.message.read_ahead or
              Delta.mailer.views.current.focus_next
    end

    def read_back
      Delta.mailer.message.read_back or
              Delta.mailer.views.current.focus_prev
    end

    def select_next
      Delta.mailer.views.current.focus_next
    end

    def select_prev
      Delta.mailer.views.current.focus_prev
    end

    def view_next
      Delta.mailer.views.current.focus_next
    end
    
    def view_prev
      Delta.mailer.views.current.focus_prev
    end

    def next_page
      @clistadj.set_value @clistadj.value + @clistadj.page_size
    end

    def prev_page
      @clistadj.set_value @clistadj.value - @clistadj.page_size
    end

    def new_mail
      Delta.mailer.edit_new
    end

    def reply_to
      orig = Delta.mailer.views.current.selected
      unless orig then
        write "no mail selected to reply"
        break
      end
      Delta.mailer.edit_reply orig
    end

    def reedit
      orig = Delta.mailer.views.current.selected
      unless orig then
        write "no mail selected to edit"
        break
      end
      Delta.mailer.reedit orig
    end

    def forward
      orig = Delta.mailer.views.current.selected
      unless orig then
        write "no mail selected to forward"
        break
      end
      Delta.mailer.forward( orig )
    end

    def send_all
      Delta.mailer.current_profile.sender.send_all_drafts
    end

    def send_selected
      ports = Delta.mailer.views.current.selection
      Delta.mailer.current_profile.sender.send_mail_from ports
    end

    def pop
      # Thread.fork {
      Delta.mailer.current_profile.popper.pop
      # }
    end

    def edit_address
    end

    def shutdown
      Gtk.main_quit
    end

    def abort
      exit! 3
    end
  
  end


  Gtk::MenuBar.module_eval "alias add append"

  class MenuMaker

    def self.generate( file, accel )
      new(file).generate(accel)
    end

    def self.debug_parse( file )
      new(file, DummyBar, DummyMenu, DummyItem).generate(nil)
    end

    def initialize( file,
                    b = Gtk::MenuBar, m = Gtk::Menu, i = Gtk::MenuItem )
      @file = file
      @barclass = b
      @menuclass = m
      @itemclass = i
      @handler = MenuActions
    end

    def generate( accel )
      @accel = accel
      File.open( @file ) {|f|
          eval_header f
          m = create_menu(f)
          register_unvisibles f
          return m
      }
    end

    def eval_header( f )
      code = ''
      while line = f.gets do
        break if /\Amenu\s/ === line
        code << line
      end
      MenuActions.module_eval code, @file, 1
    end


    def create_menu( f )
      stack = [ bar = @barclass.new ]
      indent = [0]
      lasti = nil

      while line = f.gets do
        next if line.strip.empty?
        break if /\Aunvisible/ === line

        idt = getindent(line)
        if    idt >  indent.last and lasti then
          indent.push idt
          stack.push @menuclass.new
          lasti.set_submenu stack.last
          stack.last.add lasti = new_item(line, f)

        elsif idt == indent.last then
          stack.last.add lasti = new_item(line, f)

        elsif idt <  indent.last then
          indent.pop
          stack.pop
          stack.last.add lasti = new_item(line, f)
        end
      end

      bar
    end

    def getindent( line )
      /\A(\s*)/.match(line)[1].size
    end

    RE = /\A(\w+)\s+([^(]+)(?:\s+\(([\w\-]+)\))?\z/

    def new_item( line, f )
      m = RE.match(line.strip) or
              raise ArgumentError, "wrong menu item format at line #{f.lineno}"
      act, label, key = m.to_a[1,3]
      item = @itemclass.new(label)
      bind_action item, validate_action_name(act, f.lineno) if act
      bind_key item, key if key

      item
    end


    GC_PROTECT = []

    def register_unvisibles( f )
      while line = f.gets do
        next if line.strip.empty?
        act, key = line.strip.split(/\s+/, 2)
        item = @itemclass.new(act)
        bind_action__default item, validate_action_name(act, f.lineno)
        bind_key item, key
        GC_PROTECT.push item
      end
    end


    def bind_action( item, actname )
      if item.respond_to? :signal_handler then
        if respond_to? 'bind_action__' + actname then
          __send__ 'bind_action__' + actname, item
        else
          bind_action__default item, actname
        end
      else
        item.bind_action actname
      end
    end

    def validate_action_name( name, lineno )
      (@handler.action? name or
       respond_to? 'bind_action__' + name) or
            raise ArgumentError, "wrong action name '#{name}' at line #{lineno}"
      name
    end

    def bind_action__default( item, actname )
      item.signal_handler( 'activate' ) {
          @handler.__send__ actname
      }
    end

    def bind_action__submenu( item )
    end

    def bind_action__move( item )
      item.set_submenu menu = @menuclass.new
      Delta.mailer.views.to_a.sort.each do |name, view|       # FIXME
        menu.add i = @itemclass.new(name)
        bindact_move i, view
      end
    end

    def bindact_move( item, view )
      item.signal_handler( 'activate' ) {
          @handler.move_selection_to view
      }
    end

    def bind_key( item, key )
      m = /\A(?:([CMS]{1,3})-)?(.)\z/i.match(key) or
              raise ArgumentError, "wrong key format: #{key}"
      mod = m[1]
      k = m[2]

      mask = 0
      if mod then
        mod.downcase.each_byte do |b|
          case b
          when ?c then mask |= Gdk::CONTROL_MASK
          # when ?m then mask |= Gtk::META_MASK
          when ?s then mask |= Gdk::SHIFT_MASK
          end
        end
      end
      if /[A-Z]/ === k then
        mask |= Gdk::SHIFT_MASK
        k = k.downcase
      end

      if DummyItem === item then
        item.bind_key k, mask
      else
        @accel.add k[0], mask, Gtk::AccelGroup::ACCEL_VISIBLE,
                   item, 'activate'
      end
    end

    class DummyMenu

      LABEL = '[menu]'

      def initialize
        @list = []
      end

      def add( item )
        @list.push item
      end

      def inspect( lev = 0 )
        '  ' * lev +
        type::LABEL +
        ' ' + @list.size.to_s +
        "\n" +
        @list.collect {|i| i.inspect(lev) }.join("\n")
      end

    end

    class DummyBar < DummyMenu
      LABEL = '[bar]'
    end

    class DummyItem

      def initialize( label )
        @label = label
        @sub = nil
        @key = nil
        @mask = nil
        @action = nil
      end

      def set_submenu( m )
        @sub = m
      end

      attr_reader :action

      def bind_action( act )
        @action = act
      end

      def bind_key( k, mask )
        @key = k
        @mask = mask
      end

      def inspect( lev = 0 )
        '  ' * lev +
        '[item] ' +
        '"' + @label + '"' +
        (@key ? " (#{vmask @mask}#@key)" : '') +
        (@action ? ' ' + @action : '') +
        (@sub ? "\n" + @sub.inspect(lev + 1) : '')
      end

      def vmask( mask )
        s = ''
        (mask & Gdk::CONTROL_MASK) != 0 and s << 'C'
        (mask & Gdk::SHIFT_MASK)   != 0 and s << 'S'
        s.empty? or s << '-'
        s
      end

    end

  end

end
